local jkHouses = ...

jkHouses:registerAssetProcessor("models/houses.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

jkHouses:registerAssetProcessor("models/houses_2.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})

jkHouses:registerAssetProcessor("models/house_m.fbx", {
	DataType = "BUILDING_ASSET_PROCESSOR"
})


jkHouses:registerAssetId("models/houses.fbx/Materials/transparent_roof", MOD_NS .. "TRANSPARENT_ROOF")
jkHouses:override({
    Id = MOD_NS .. "TRANSPARENT_ROOF",
	HasTransparency = true,
	HasAlphaTest = true,
	BackFaceVisible = true,
	IsLighted = true,
	HasShadow = true
})

jkHouses:registerAssetId("models/houses.fbx/Materials/stone_column", MOD_NS .. "STONE_COLUMN_MATERIAL")
jkHouses:override({
    Id = MOD_NS .. "STONE_COLUMN_MATERIAL",
	BackFaceVisible = false,
	HasShadow = false,
})

jkHouses:registerAssetId("models/houses.fbx/Prefab/DummyPart", MOD_NS .. "PREFAB_DUMMY")
jkHouses:registerPrefabComponent("models/houses.fbx/Prefab/DummyPart", {
	DataType = "COMP_GROUNDED"
})

jkHouses:register({
	DataType = "BUILDING_PART",
	Id = MOD_NS .. "DUMMY_PART",
	Description = "",
	Capacity = 0,
	Category = "CORE",
	ConstructionVisual = MOD_NS .. "PREFAB_DUMMY",
	ConstructorData = {
		DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
		CoreObjectPrefab = MOD_NS .. "PREFAB_DUMMY"
	},
	Cost = {
		UpkeepCost = {
		},
		RessourcesNeeded = {
			{ Resource = "WOOD", Quantity = 10 }
		}
	}
})


function registerHouseFunction(name, upgradedToPart, lvl)
	lvl = lvl or "HOUSING_LEVEL_1"
	local houseStatusList =  {
		{
			DataType = "HOUSE_STATUS",
			RequiredDesirability = DESIRABILITY_LEVEL.LOW,
			ResourceGenerated = "HOUSING_LEVEL_1",
			HouseDensityPartList = { MOD_NS .. upgradedToPart }
		},
		{
			DataType = "HOUSE_STATUS",
			RequiredDesirability = DESIRABILITY_LEVEL.AVERAGE,
			ResourceGenerated = "HOUSING_LEVEL_1",
			HouseDensityPartList = { MOD_NS .. upgradedToPart }
		}
	}
	if lvl == "HOUSING_LEVEL_2" then
		houseStatusList = {
			{
				DataType = "HOUSE_STATUS",
				RequiredDesirability = DESIRABILITY_LEVEL.AVERAGE,
				ResourceGenerated = "HOUSING_LEVEL_2",
				HouseDensityPartList = { MOD_NS .. upgradedToPart }
			},
			{
				DataType = "HOUSE_STATUS",
				RequiredDesirability = DESIRABILITY_LEVEL.VERY_HIGH,
				ResourceGenerated = "HOUSING_LEVEL_2",
				HouseDensityPartList = { MOD_NS .. upgradedToPart }
			}
		}
	end

	local setupId = MOD_NS .. name .. "_SETUP"
	jkHouses:register({
		DataType= "HOUSE_SETUP",
		Id = setupId,
		HouseStatusList = houseStatusList
	})

	local functionName = MOD_NS .. name .. "_FUNCTION"
	jkHouses:register({
		DataType = "BUILDING_FUNCTION_HOUSE",
		Id = functionName,
		RelatedZone = "RESIDENTIAL",
		RelatedDesirability = "DESIRABILITY_RESIDENTIAL",
		AssetHouseSetup =  setupId,
		TimeToUpgradeDensity = 1
	})
	return functionName
end



local HOUSES = "models/houses.fbx/Prefab/";
local HOUSES_2 = "models/houses_2.fbx/Prefab/";

function registerHousePart(path, name, functionName, dirtGroundSize)
	__registerHousePart(HOUSES .. path, name, functionName, dirtGroundSize)
end

function __registerBasicHousePart(id, name, functionName, groundSize, capacity)
	jkHouses:register({
		DataType = "BUILDING_PART",
		Id = id,
		Description = "",
		Name = MOD_NS .. name .."_NAME",
		Capacity = capacity,
		IsMovableWhenBuilt = true,
		Category = "CORE",
		ConstructionVisual = MOD_NS .. "PREFAB_" .. name,
		AssetBuildingFunction = functionName,
		BuildingZone = { 
			ZoneEntryList = {
				{ 
					Polygon = polygon.createRectangle( groundSize, {0, 0} ),
					Type = { DEFAULT = true, NAVIGABLE = false, GRASS_CLEAR = true } 
				}
			}
		},
		ConstructorData = {
			DataType = "BUILDING_CONSTRUCTOR_DEFAULT",
			CoreObjectPrefab = MOD_NS .. "PREFAB_" .. name
		},
		Cost = {
			UpkeepCost = {
			},
			RessourcesNeeded = {
				{ Resource = "WOOD", Quantity = 10 }
			}
		}
	})
end

function __registerHousePart(fbxPath, name, functionName, dirtGroundSize)
	local groundSize = dirtGroundSize or {4, 6}
	jkHouses:registerAssetId(fbxPath, MOD_NS .. "PREFAB_" .. name)
	jkHouses:registerPrefabComponent(fbxPath, {
		DataType = "COMP_DIRT_RECTANGLE",
		Size = groundSize
	})
	jkHouses:registerPrefabComponent(fbxPath, {
		DataType = "COMP_BUILDING_PART",
	})
	jkHouses:registerPrefabComponent(fbxPath, {
		DataType = "COMP_GROUNDED"
	})
	-- jkHouses:registerPrefabComponent(fbxPath, {
	-- 	DataType = "COMP_TRACER"
	-- })

	__registerBasicHousePart(MOD_NS .. name .."_PART", name, functionName, groundSize, 4)
	__registerBasicHousePart(MOD_NS .. name .."_0_PART", name, functionName, groundSize, 0)
end

function registerHouse(path, name, functionName, dirtGroundSize)
	registerHousePart(path, name, functionName, dirtGroundSize)
	jkHouses:register({
		DataType = "BUILDING",
		Id = MOD_NS .. name,
		Name = MOD_NS .. name .."_NAME",
		BuildingType = BUILDING_TYPE.HOUSING,
		AssetCoreBuildingPart = MOD_NS .. name .."_PART",
		IsDestructible = true,
		IsEditable = true,
		IsClearTrees = true
	})
end

local lvl_2 = registerHouseFunction("HOUSING_LEVEL_2", "DUMMY_PART", "HOUSING_LEVEL_2")
local lvl_1 = registerHouseFunction("HOUSING_LEVEL_1", "DUMMY_PART", "HOUSING_LEVEL_1")

registerHouse("HouseAPart", "HOUSE_A", lvl_1, {12, 6})
registerHouse("HouseBPart", "HOUSE_B", lvl_1, {7, 6})
registerHouse("HouseCPart", "HOUSE_C", lvl_1, {8, 6})
registerHouse("HouseDPart", "HOUSE_D", lvl_2)
registerHouse("HouseEPart", "HOUSE_E", lvl_1, {5, 5})
registerHouse("HouseFPart", "HOUSE_F", lvl_2, {5, 6})
registerHouse("HouseGPart", "HOUSE_G", lvl_2, {7, 5.5})
registerHouse("HouseHPart", "HOUSE_H", lvl_1, {6, 6})
registerHouse("HouseIPart", "HOUSE_I", lvl_2, {6, 6})
registerHouse("HouseJPart", "HOUSE_J", lvl_2, {6, 6})
registerHouse("HouseKPart", "HOUSE_K", lvl_2)
registerHouse("HouseLPart", "HOUSE_L", lvl_2)

local HOUSE_M = "models/house_m.fbx/Prefab/HouseMPart"
__registerHousePart(HOUSE_M, "HOUSE_M", lvl_2)
__registerHousePart(HOUSES .. "HouseNPart", "HOUSE_N", lvl_2, {4, 4})
__registerHousePart(HOUSES .. "HouseOPart", "HOUSE_O", lvl_1, {5, 6})
__registerHousePart(HOUSES .. "HousePPart", "HOUSE_P", lvl_2, {7, 7})
__registerHousePart(HOUSES_2 .. "HouseAPart", "HOUSE_AA", lvl_2)
__registerHousePart(HOUSES_2 .. "HouseHPart", "HOUSE_AH", lvl_1, {9, 6.5})

jkHouses:configurePrefabFlagList(HOUSE_M, { "PLATFORM" })
jkHouses:configurePrefabFlagList("models/houses.fbx/Prefab/HouseFPart", { "PLATFORM" })

function __registerGroundedComponent(fbxPath, name)
	jkHouses:registerAssetId(fbxPath, MOD_NS .. "PREFAB_" .. name .. "_CMP")
	jkHouses:registerPrefabComponent(fbxPath, {
		DataType = "COMP_GROUNDED"
	})
end

function registerGroundedComponent(path, name)
	__registerGroundedComponent("models/houses.fbx/Prefab/" .. path, name)
end

registerGroundedComponent("HouseAPart/HouseADoorPart", "HOUSE_A_DOOR")
registerGroundedComponent("HouseBPart/ToiletCabinPart", "HOUSE_B_TOILET")
registerGroundedComponent("HouseCPart/BarnPart", "HOUSE_C_BARN")
registerGroundedComponent("HouseDPart/HouseDDoorPart", "HOUSE_D_DOOR")
registerGroundedComponent("HouseEPart/HouseEDoorPart", "HOUSE_E_DOOR")
registerGroundedComponent("HouseFPart/HouseFDoorPart", "HOUSE_F_DOOR")
registerGroundedComponent("HouseGPart/HouseGDoorPart", "HOUSE_G_DOOR")
registerGroundedComponent("HouseIPart/HouseIDoorPart", "HOUSE_I_DOOR")
registerGroundedComponent("HouseIPart/HouseIDoor2Part", "HOUSE_I_2_DOOR")
registerGroundedComponent("HouseJPart/HouseJDoorPart", "HOUSE_J_DOOR")
registerGroundedComponent("HouseKPart/HouseKDoorPart", "HOUSE_K_DOOR")
registerGroundedComponent("HouseLPart/HouseLDoorPart", "HOUSE_L_DOOR")
registerGroundedComponent("HousePPart/HousePDoorPart", "HOUSE_P_DOOR")

__registerGroundedComponent("models/houses_2.fbx/Prefab/HouseHPart/HouseHDoorPart", "HOUSE_AH_DOOR")


function registerSmokeEmitter(path, name)
	__registerSmokeEmitter("models/houses.fbx/Prefab/" .. path, name)
end

function __registerSmokeEmitter(houseFbxPath, name)
	local smokeEmitterPrefab = MOD_NS .. "HOUSE_" .. name .. " _SMOKE_EMITTER_PREFAB"
	local emitterPath = houseFbxPath .. "/SmokeEmitter" .. name
	jkHouses:registerAssetId(emitterPath, smokeEmitterPrefab)
	jkHouses:registerPrefabComponent(houseFbxPath, {
		DataType = "COMP_BUILDING_PART",
		FeedbackComponentListToActivate = {
			{
				smokeEmitterPrefab, "COMP_PARTICLE_EMITTER_TOGGLE"
			}
		}
	})
	jkHouses:registerPrefabComponent(emitterPath, {
		DataType = "COMP_PARTICLE_EMITTER_TOGGLE",
		Enabled = false
	})
	jkHouses:registerPrefabComponent(emitterPath, {
		DataType = "COMP_PARTICLE_EMITTER",
		ParticleSystem = "PARTICLE_SYSTEM_HOUSING_SMOKE",
		IsPlaying = false,
		IsEmitting = false
	})
end

registerSmokeEmitter("HouseAPart", "A")
registerSmokeEmitter("HouseBPart", "B")
registerSmokeEmitter("HouseDPart", "D")
registerSmokeEmitter("HouseEPart", "E")
registerSmokeEmitter("HouseFPart", "F")
registerSmokeEmitter("HouseGPart", "G")
registerSmokeEmitter("HouseIPart", "I")
registerSmokeEmitter("HouseJPart", "J")
registerSmokeEmitter("HouseKPart", "K")
registerSmokeEmitter("HouseLPart", "L")
__registerSmokeEmitter(HOUSE_M, "M")
registerSmokeEmitter("HouseOPart", "O")
registerSmokeEmitter("HousePPart", "P")
__registerSmokeEmitter(HOUSES_2 .. "HouseHPart", "AH")
__registerSmokeEmitter(HOUSES_2 .. "HouseAPart", "AA")

jkHouses:register({
	DataType = "BUILDING",
	Id = MOD_NS .. "RESIDENTIAL_AREA",
	Name = MOD_NS .. "RESIDENTIAL_AREA_NAME",
	BuildingType = BUILDING_TYPE.MONUMENT,
	AssetCoreBuildingPart = "BUILDING_PART_MONUMENT_POLE",
	BuildingPartSetList = {
		{
			Name = "COMMON_HOUSES",
			BuildingPartList = {
				MOD_NS .. "HOUSE_A_PART",
				MOD_NS .. "HOUSE_B_PART",
				MOD_NS .. "HOUSE_C_PART",
				MOD_NS .. "HOUSE_E_PART",
				MOD_NS .. "HOUSE_H_PART",
				MOD_NS .. "HOUSE_AH_PART",
				MOD_NS .. "HOUSE_O_PART",
			}
		}, {
			Name = "CITIZEN_HOUSES",
			BuildingPartList = {
				MOD_NS .. "HOUSE_D_PART",
				MOD_NS .. "HOUSE_F_PART",
				MOD_NS .. "HOUSE_G_PART",
				MOD_NS .. "HOUSE_I_PART",
				MOD_NS .. "HOUSE_J_PART",
				MOD_NS .. "HOUSE_K_PART",
				MOD_NS .. "HOUSE_L_PART",
				MOD_NS .. "HOUSE_M_PART",
				MOD_NS .. "HOUSE_AA_PART",
				MOD_NS .. "HOUSE_N_PART",
				MOD_NS .. "HOUSE_P_PART",
			},
		}, {
				BuildingPartList = {
					MOD_NS .. "FOUNTAIN_PART",
				}
		}
	},
	IsDestructible = true,
	IsEditable = true,
	IsClearTrees = true
})